@@ -25,6 +25,9 @@ export const DEFAULT_CHE_OPERATOR_IMAGE_NAME = 'registry.redhat.io/codeready-wor
 // This image should be updated manually when needed.
 // Repository location: https://github.com/che-dockerfiles/che-cert-manager-ca-cert-generator-image
 export const CA_CERT_GENERATION_JOB_IMAGE = 'quay.io/eclipse/che-cert-manager-ca-cert-generator:671342c'
+export const INDEX_IMG = 'quay.io/eclipse/eclipse-che-openshift-opm-catalog:next'
+export const DEV_WORKSPACE_NEXT_CATALOG_SOURCE_IMAGE = 'quay.io/devfile/devworkspace-operator-index:next'
+export const DEV_WORKSPACE_STABLE_CATALOG_SOURCE_IMAGE = 'quay.io/devfile/devworkspace-operator-index:release'
 
 export const NEXT_TAG = 'next'
 
@@ -44,8 +47,8 @@ export const LEGACY_CHE_NAMESPACE = 'workspaces'
 export const DEFAULT_CHE_OLM_PACKAGE_NAME = 'codeready-workspaces'
 export const DEFAULT_CHE_OPERATOR_SUBSCRIPTION_NAME = 'codeready-subscription'
 export const OPERATOR_GROUP_NAME = 'codeready-operator-group'
-export const CVS_PREFIX = 'crwoperator'
-export const DEVWORKSPACE_CVS_PREFIX = 'crwoperator'
+export const CSV_PREFIX = 'eclipse-che'
+export const DEVWORKSPACE_CSV_PREFIX = 'devworkspace-operator'
 // OLM channels
 export const OLM_STABLE_CHANNEL_NAME = 'latest'
 export const OLM_STABLE_CHANNEL_STARTING_CSV_TEMPLATE = 'crwoperator.v{{VERSION}}'
@@ -60,13 +63,15 @@ export const DEFAULT_OPENSHIFT_OPERATORS_NS_NAME = 'openshift-operators'
 // OLM catalogs
 export const CUSTOM_CATALOG_SOURCE_NAME = 'codeready-custom-catalog-source'
 export const NEXT_CATALOG_SOURCE_NAME = 'eclipse-che-preview'
+export const NEXT_CATALOG_SOURCE_DEV_WORKSPACE_OPERATOR = 'custom-devworkspace-operator-catalog'
+export const STABLE_CATALOG_SOURCE_DEV_WORKSPACE_OPERATOR = 'stable-custom-devworkspace-operator-catalog'
 export const DEFAULT_OLM_SUGGESTED_NAMESPACE = 'openshift-workspaces'
 export const KUBERNETES_OLM_CATALOG = 'operatorhubio-catalog'
 export const OPENSHIFT_OLM_CATALOG = 'redhat-operators'
 
 // Documentation links
-export const DOC_LINK = 'https://access.redhat.com/documentation/en-us/red_hat_codeready_workspaces/2.14/'
-export const DOC_LINK_RELEASE_NOTES = 'https://access.redhat.com/documentation/en-us/red_hat_codeready_workspaces/2.14/html/release_notes_and_known_issues/index'
+export const DOC_LINK = 'https://access.redhat.com/documentation/en-us/red_hat_codeready_workspaces/2.15/'
+export const DOC_LINK_RELEASE_NOTES = 'https://access.redhat.com/documentation/en-us/red_hat_codeready_workspaces/2.15/html/release_notes_and_known_issues/index'
 export const DOCS_LINK_INSTALL_RUNNING_CHE_LOCALLY = 'https://www.eclipse.org/che/docs/che-7/installation-guide/installing-che-locally/'
 export const DOCS_LINK_IMPORT_CA_CERT_INTO_BROWSER = 'https://www.eclipse.org/che/docs/che-7/end-user-guide/importing-certificates-to-browsers/'
 export const DOCS_LINK_HOW_TO_ADD_IDENTITY_PROVIDER_OS4 = 'https://docs.openshift.com/container-platform/latest/authentication/understanding-identity-provider.html#identity-provider-overview_understanding-identity-provider'
@@ -99,4 +104,12 @@ export const CHE_CLUSTER_BACKUP_KIND_PLURAL = 'checlusterbackups'
 export const CHE_CLUSTER_RESTORE_CRD = 'checlusterrestores.org.eclipse.che'
 export const CHE_CLUSTER_RESTORE_KIND_PLURAL = 'checlusterrestores'
 
+export const DEVFILE_WORKSPACE_API_GROUP = 'workspace.devfile.io'
+export const DEVFILE_WORKSPACE_API_VERSION = 'v1alpha2'
+export const DEVFILE_WORKSPACE_KIND_PLURAL = 'devworkspaces'
+
+export const DEVFILE_WORKSPACE_ROUTINGS_API_GROUP = 'controller.devfile.io'
+export const DEVFILE_WORKSPACE_ROUTINGS_VERSION = 'v1alpha1'
+export const DEVFILE_WORKSPACE_ROUTINGS_KIND_PLURAL = 'devworkspaceroutings'
+
 export const DEFAULT_CHE_TLS_SECRET_NAME = 'che-tls'
