@@ -11,11 +11,10 @@
  */
 
 import * as Listr from 'listr'
-
 import { CheHelper } from '../../api/che'
 import { KubeHelper } from '../../api/kube'
 import { OpenShiftHelper } from '../../api/openshift'
-import { DEFAULT_DEV_WORKSPACE_CONTROLLER_NAMESPACE, DEFAULT_OPENSHIFT_OPERATORS_NS_NAME, DEVWORKSPACE_CVS_PREFIX } from '../../constants'
+import { DEFAULT_DEV_WORKSPACE_CONTROLLER_NAMESPACE, DEVFILE_WORKSPACE_API_GROUP, DEVFILE_WORKSPACE_API_VERSION, DEVFILE_WORKSPACE_KIND_PLURAL, DEVFILE_WORKSPACE_ROUTINGS_API_GROUP, DEVFILE_WORKSPACE_ROUTINGS_KIND_PLURAL, DEVFILE_WORKSPACE_ROUTINGS_VERSION } from '../../constants'
 import { CertManagerTasks } from '../component-installers/cert-manager'
 
 /**
@@ -31,6 +30,7 @@ export class DevWorkspaceTasks {
   protected certManagerTask: CertManagerTasks
 
   protected devWorkspaceServiceAccount = 'devworkspace-controller-serviceaccount'
+  protected devWorkspaceWebhookServiceAccount = 'devworkspace-webhook-server'
 
   // DevWorkspace Controller Roles
   protected devWorkspaceLeaderElectionRole = 'devworkspace-controller-leader-election-role'
@@ -54,10 +54,14 @@ export class DevWorkspaceTasks {
 
   protected devWorkspaceRoleBinding = 'devworkspace-controller-rolebinding'
 
-  protected devWorkspaceWebhookServerClusterRolebinding = 'devworkspace-webhook-server'
+  protected devWorkspaceWebhookServerClusterRole = 'devworkspace-webhook-server'
 
   // Deployment names
   protected deploymentName = 'devworkspace-controller-manager'
+  protected deploymentWebhookName = 'devworkspace-webhook-server'
+
+  // Services
+  protected serviceWebhookName = 'devworkspace-webhookserver'
 
   // ConfigMap names
   protected devWorkspaceConfigMap = 'devworkspace-controller-configmap'
@@ -75,6 +79,8 @@ export class DevWorkspaceTasks {
 
   protected workspaceRoutingsCrdName = 'devworkspaceroutings.controller.devfile.io'
 
+  protected devWorkspaceConfigCrdName = 'devworkspaceoperatorconfigs.controller.devfile.io'
+
   protected webhooksName = 'controller.devfile.io'
 
   // Web Terminal Operator constants
@@ -104,147 +110,101 @@ export class DevWorkspaceTasks {
     ]
   }
 
-  async isDevWorkspaceInstalledViaOLM(): Promise<boolean> {
-    const IsPreInstalledOLM = await this.kubeHelper.isPreInstalledOLM()
-    if (!IsPreInstalledOLM) {
-      return false
-    }
-    const csvAll = await this.kubeHelper.getClusterServiceVersions(DEFAULT_OPENSHIFT_OPERATORS_NS_NAME)
-    const devWorkspaceCSVs = csvAll.items.filter(csv => csv.metadata.name!.startsWith(DEVWORKSPACE_CVS_PREFIX))
-    return devWorkspaceCSVs.length > 0
-  }
-
   /**
-   * Returns list of tasks which uninstall dev-workspace.
+   * Returns list of tasks which uninstall dev-workspace operator.
    */
-  getUninstallTasks(): ReadonlyArray<Listr.ListrTask> {
+  deleteResourcesTasks(): ReadonlyArray<Listr.ListrTask> {
     return [
       {
-        title: 'Check DevWorkspace OLM installation',
-        task: async (ctx: any, task: any) => {
-          ctx.isDevWorkspaceInstalledViaOLM = Boolean(await this.isDevWorkspaceInstalledViaOLM())
-
-          task.title = await `${task.title} ...${ctx.isDevWorkspaceInstalledViaOLM ? 'Installed' : 'Not installed'}`
-        },
-      },
-      {
-        title: 'Delete all DevWorkspace Controller deployments',
+        title: 'Delete all Dev Workspace Controller deployments',
         task: async (_ctx: any, task: any) => {
           await this.kubeHelper.deleteAllDeployments(DEFAULT_DEV_WORKSPACE_CONTROLLER_NAMESPACE)
-          task.title = await `${task.title}...OK`
+          task.title = `${task.title}...[OK]`
         },
       },
       {
-        title: 'Delete all DevWorkspace Controller services',
+        title: 'Delete all Dev Workspace Controller services',
         task: async (_ctx: any, task: any) => {
           await this.kubeHelper.deleteAllServices(DEFAULT_DEV_WORKSPACE_CONTROLLER_NAMESPACE)
-          task.title = await `${task.title}...OK`
+          task.title = `${task.title}...[OK]`
         },
       },
       {
-        title: 'Delete all DevWorkspace Controller routes',
+        title: 'Delete all Dev Workspace Controller routes',
         enabled: (ctx: any) => !ctx.isOpenShift,
         task: async (_ctx: any, task: any) => {
           await this.kubeHelper.deleteAllIngresses(DEFAULT_DEV_WORKSPACE_CONTROLLER_NAMESPACE)
-          task.title = await `${task.title}...OK`
+          task.title = `${task.title}...[OK]`
         },
       },
       {
-        title: 'Delete all DevWorkspace Controller routes',
+        title: 'Delete all Dev Workspace Controller routes',
         enabled: (ctx: any) => ctx.isOpenShift,
         task: async (_ctx: any, task: any) => {
           await this.openShiftHelper.deleteAllRoutes(DEFAULT_DEV_WORKSPACE_CONTROLLER_NAMESPACE)
-          task.title = await `${task.title}...OK`
+          task.title = `${task.title}...[OK]`
         },
       },
       {
-        title: 'Delete DevWorkspace Controller configmaps',
+        title: 'Delete Dev Workspace Controller configmaps',
         task: async (_ctx: any, task: any) => {
           await this.kubeHelper.deleteConfigMap(this.devWorkspaceConfigMap, DEFAULT_DEV_WORKSPACE_CONTROLLER_NAMESPACE)
 
-          task.title = await `${task.title}...OK`
+          task.title = `${task.title}...[OK]`
         },
       },
       {
-        title: 'Delete DevWorkspace Controller ClusterRoleBindings',
-        task: async (ctx: any, task: any) => {
+        title: 'Delete Dev Workspace Controller ClusterRoleBindings',
+        task: async (_ctx: any, task: any) => {
           await this.kubeHelper.deleteClusterRoleBinding(this.devWorkspaceRoleBinding)
           await this.kubeHelper.deleteClusterRoleBinding(this.devworkspaceProxyClusterRoleBinding)
-          if (!ctx.isDevWorkspaceInstalledViaOLM) {
-            await this.kubeHelper.deleteClusterRoleBinding(this.devWorkspaceWebhookServerClusterRolebinding)
-          }
 
-          task.title = await `${task.title}...OK`
+          task.title = `${task.title}...[OK]`
         },
       },
       {
-        title: 'Delete DevWorkspace Controller role',
+        title: 'Delete Dev Workspace Controller role',
         task: async (_ctx: any, task: any) => {
           await this.kubeHelper.deleteRole(this.devWorkspaceLeaderElectionRole, DEFAULT_DEV_WORKSPACE_CONTROLLER_NAMESPACE)
 
-          task.title = await `${task.title}...OK`
+          task.title = `${task.title}...[OK]`
         },
       },
       {
-        title: 'Delete DevWorkspace Controller roleBinding',
+        title: 'Delete Dev Workspace Controller roleBinding',
         task: async (_ctx: any, task: any) => {
           await this.kubeHelper.deleteRoleBinding(this.devWorkspaceLeaderElectionRoleBinding, DEFAULT_DEV_WORKSPACE_CONTROLLER_NAMESPACE)
 
-          task.title = await `${task.title}...OK`
+          task.title = `${task.title}...[OK]`
         },
       },
       {
-        title: 'Delete DevWorkspace Controller cluster roles',
-        task: async (ctx: any, task: any) => {
-          if (!ctx.isDevWorkspaceInstalledViaOLM) {
-            await this.kubeHelper.deleteClusterRole(this.devWorkspaceEditWorkspaceClusterRole)
-            await this.kubeHelper.deleteClusterRole(this.devWorkspaceViewWorkspaceClusterRole)
-            await this.kubeHelper.deleteClusterRole(this.devWorkspaceClusterRoleWebhook)
-          }
-
+        title: 'Delete Dev Workspace Controller cluster roles',
+        task: async (_ctx: any, task: any) => {
+          await this.kubeHelper.deleteClusterRole(this.devWorkspaceEditWorkspaceClusterRole)
+          await this.kubeHelper.deleteClusterRole(this.devWorkspaceViewWorkspaceClusterRole)
           await this.kubeHelper.deleteClusterRole(this.devworkspaceProxyClusterRole)
           await this.kubeHelper.deleteClusterRole(this.devworkspaceClusterRole)
 
-          task.title = await `${task.title}...OK`
+          task.title = `${task.title}...[OK]`
         },
       },
       {
-        title: 'Delete DevWorkspace Controller service account',
+        title: 'Delete Dev Workspace Controller service account',
         task: async (_ctx: any, task: any) => {
           await this.kubeHelper.deleteServiceAccount(this.devWorkspaceServiceAccount, DEFAULT_DEV_WORKSPACE_CONTROLLER_NAMESPACE)
 
-          task.title = await `${task.title}...OK`
+          task.title = `${task.title}...[OK]`
         },
       },
       {
-        title: 'Delete DevWorkspace Controller self-signed certificates',
+        title: 'Delete Dev Workspace Controller self-signed certificates',
         enabled: async (ctx: any) => !ctx.IsOpenshift,
         task: async (_ctx: any, task: any) => {
           await this.kubeHelper.deleteNamespacedCertificate(this.devWorkspaceCertificate, 'v1', DEFAULT_DEV_WORKSPACE_CONTROLLER_NAMESPACE)
           await this.kubeHelper.deleteNamespacedIssuer(this.devWorkspaceCertIssuer, 'v1', DEFAULT_DEV_WORKSPACE_CONTROLLER_NAMESPACE)
 
-          task.title = await `${task.title}...OK`
-        },
-      },
-      {
-        title: 'Delete DevWorkspace Controller webhooks configurations',
-        enabled: ctx => !ctx.isDevWorkspaceInstalledViaOLM,
-        task: async (_ctx: any, task: any) => {
-          await this.kubeHelper.deleteMutatingWebhookConfiguration(this.webhooksName)
-          await this.kubeHelper.deleteValidatingWebhookConfiguration(this.webhooksName)
-
-          task.title = await `${task.title} ...OK`
-        },
-      },
-      {
-        title: 'Delete DevWorkspace Controller CRDs',
-        enabled: ctx => !ctx.isDevWorkspaceInstalledViaOLM,
-        task: async (_ctx: any, task: any) => {
-          await this.kubeHelper.deleteCrd(this.devWorkspacesCrdName)
-          await this.kubeHelper.deleteCrd(this.devWorkspaceTemplatesCrdName)
-          await this.kubeHelper.deleteCrd(this.workspaceRoutingsCrdName)
-
-          task.title = await `${task.title}...OK`
+          task.title = `${task.title}...[OK]`
         },
       },
       {
@@ -254,7 +214,86 @@ export class DevWorkspaceTasks {
           if (namespaceExist) {
             await this.kubeHelper.deleteNamespace(DEFAULT_DEV_WORKSPACE_CONTROLLER_NAMESPACE)
           }
-          task.title = `${task.title}...OK`
+          task.title = `${task.title}...[OK]`
+        },
+      },
+    ]
+  }
+
+  deleteDevOperatorCRsAndCRDsTasks(): ReadonlyArray<Listr.ListrTask> {
+    return [
+      {
+        title: `Delete ${DEVFILE_WORKSPACE_API_GROUP}/${DEVFILE_WORKSPACE_API_VERSION} resources`,
+        task: async (_ctx: any, task: any) => {
+          await this.kubeHelper.deleteAllCustomResources(DEVFILE_WORKSPACE_API_GROUP, DEVFILE_WORKSPACE_API_VERSION, DEVFILE_WORKSPACE_KIND_PLURAL)
+          task.title = `${task.title}...[OK]`
+        },
+      },
+      {
+        title: `Delete ${DEVFILE_WORKSPACE_ROUTINGS_API_GROUP}/${DEVFILE_WORKSPACE_ROUTINGS_VERSION} resources`,
+        task: async (_ctx: any, task: any) => {
+          await this.kubeHelper.deleteAllCustomResources(DEVFILE_WORKSPACE_ROUTINGS_API_GROUP, DEVFILE_WORKSPACE_ROUTINGS_VERSION, DEVFILE_WORKSPACE_ROUTINGS_KIND_PLURAL)
+          task.title = `${task.title}...[OK]`
+        },
+      },
+      {
+        title: 'Delete Dev Workspace CRDs',
+        task: async (_ctx: any, task: any) => {
+          await this.kubeHelper.deleteCrd(this.devWorkspacesCrdName)
+          await this.kubeHelper.deleteCrd(this.devWorkspaceTemplatesCrdName)
+          await this.kubeHelper.deleteCrd(this.workspaceRoutingsCrdName)
+          await this.kubeHelper.deleteCrd(this.devWorkspaceConfigCrdName)
+
+          task.title = await `${task.title}...[OK]`
+        },
+      },
+    ]
+  }
+
+  deleteDevWorkspaceWebhooksTasks(namespace: string): ReadonlyArray<Listr.ListrTask> {
+    return [
+      {
+        title: 'Delete Dev Workspace webhooks deployment',
+        task: async (_ctx: any, task: any) => {
+          await this.kubeHelper.deleteDeployment(namespace, this.deploymentWebhookName)
+          task.title = `${task.title}...[OK]`
+        },
+      },
+      {
+        title: 'Delete all Dev Workspace webhooks services',
+        task: async (_ctx: any, task: any) => {
+          await this.kubeHelper.deleteService(this.serviceWebhookName, namespace)
+          task.title = `${task.title}...[OK]`
+        },
+      },
+      {
+        title: 'Delete Dev Workspace webhook Cluster RoleBinding',
+        task: async (_ctx: any, task: any) => {
+          await this.kubeHelper.deleteClusterRoleBinding(this.devWorkspaceWebhookServerClusterRole)
+          task.title = `${task.title}...[OK]`
+        },
+      },
+      {
+        title: 'Delete Dev Workspace webhook Cluster Role',
+        task: async (_ctx: any, task: any) => {
+          await this.kubeHelper.deleteClusterRole(this.devWorkspaceWebhookServerClusterRole)
+          task.title = `${task.title}...[OK]`
+        },
+      },
+      {
+        title: 'Delete DevWorkspace webhooks service account',
+        task: async (_ctx: any, task: any) => {
+          await this.kubeHelper.deleteServiceAccount(this.devWorkspaceWebhookServiceAccount, namespace)
+          task.title = `${task.title}...[OK]`
+        },
+      },
+      {
+        title: 'Delete Dev Workspace webhooks configurations',
+        enabled: ctx => !ctx.isOLMStableDevWorkspaceOperator && !ctx.devWorkspacesPresent,
+        task: async (_ctx: any, task: any) => {
+          await this.kubeHelper.deleteMutatingWebhookConfiguration(this.webhooksName)
+          await this.kubeHelper.deleteValidatingWebhookConfiguration(this.webhooksName)
+          task.title = `${task.title} ...[OK]`
         },
       },
     ]
