@@ -1,21 +1,26 @@
-/*********************************************************************
- * Copyright (c) 2020 Red Hat, Inc.
- *
+/**
+ * Copyright (c) 2019-2021 Red Hat, Inc.
  * This program and the accompanying materials are made
  * available under the terms of the Eclipse Public License 2.0
  * which is available at https://www.eclipse.org/legal/epl-2.0/
  *
  * SPDX-License-Identifier: EPL-2.0
- **********************************************************************/
+ *
+ * Contributors:
+ *   Red Hat, Inc. - initial API and implementation
+ */
+
 import { Octokit } from '@octokit/rest'
 import * as execa from 'execa'
+import { spawn } from 'child_process'
 import * as fs from 'fs-extra'
+import * as semver from 'semver'
 
 import { CheHelper } from '../../src/api/che'
 import { CheGithubClient, TagInfo } from '../../src/api/github-client'
 import { KubeHelper } from '../../src/api/kube'
 import { OpenShiftHelper } from '../../src/api/openshift'
-import { DEFAULT_OLM_SUGGESTED_NAMESPACE } from '../../src/constants'
+import { CHE_CLUSTER_API_GROUP, CHE_CLUSTER_API_VERSION, CHE_CLUSTER_BACKUP_KIND_PLURAL, DEFAULT_OLM_SUGGESTED_NAMESPACE } from '../../src/constants'
 
 // Fields which crwctl returns for workspace:list commands
 interface WorkspaceInfo {
@@ -65,6 +70,10 @@ export class E2eHelper {
     return `${process.cwd()}/bin/run`
   }
 
+  /**
+   * Runs given command and returns its output (including error stream if any).
+   * See also runCliCommandVerbose for debug purposes.
+   */
   async runCliCommand(command: string, args?: string[], printOutput = true): Promise<string> {
     if (printOutput) {
       // tslint:disable-next-line: no-console
@@ -87,6 +96,37 @@ export class E2eHelper {
     return stdout
   }
 
+  /**
+   * Runs given cli command.
+   * Unlike runCliCommand, it listens to the command output and prints it immediatly.
+   * This function is useful for debigging, especially if the command hungs at some point.
+   */
+  async runCliCommandVerbose(command: string, args?: string[]): Promise<void> {
+    const argsString = args ? args.join(' ') : ''
+    // tslint:disable-next-line: no-console
+    console.log(`Running command ${command} ${argsString}`)
+
+    return new Promise(resolve => {
+      const process = spawn(command, args)
+
+      process.stdout.on('data', data => {
+        // tslint:disable-next-line: no-console
+        console.log(`[output] ${data}`)
+      })
+      process.stderr.on('data', data => {
+        // tslint:disable-next-line: no-console
+        console.log(`[error] ${data}`)
+      })
+
+      process.on('exit', code => {
+        // tslint:disable-next-line: no-console
+        console.log(`Command ${command} ${argsString} exited with code ${code}`)
+
+        resolve()
+      })
+    })
+  }
+
   // Return an array with all user workspaces
   // async getAllWorkspaces(isOpenshiftPlatformFamily: string): Promise<chetypes.workspace.Workspace[]> {
   private async getAllWorkspaces(): Promise<WorkspaceInfo[]> {
@@ -165,7 +205,7 @@ export class E2eHelper {
 
   // Return ingress and protocol from minikube platform
   async K8SHostname(ingressName: string, namespace: string): Promise<string> {
-    if (await this.kubeHelper.ingressExist(ingressName, namespace)) {
+    if (await this.kubeHelper.isIngressExist(ingressName, namespace)) {
       const protocol = await this.kubeHelper.getIngressProtocol(ingressName, namespace)
       const hostname = await this.kubeHelper.getIngressHost(ingressName, namespace)
 
@@ -215,6 +255,28 @@ export class E2eHelper {
     throw new Error(`Che server image tag ${tag} has not appeared in ${timeoutMs / 1000}s `)
   }
 
+  async waitForSuccessfulBackup(backupCrName: string, timeoutMs: number): Promise<void> {
+    const delayMs = 5 * 1000
+
+    let totalTimeMs = 0
+    while (totalTimeMs < timeoutMs) {
+      const backupCr = await this.kubeHelper.getCustomResource(NAMESPACE, backupCrName, CHE_CLUSTER_API_GROUP, CHE_CLUSTER_API_VERSION, CHE_CLUSTER_BACKUP_KIND_PLURAL)
+      if (backupCr && backupCr.status && backupCr.status.state) {
+        if (backupCr.status.state === 'Succeeded') {
+          // Backup successfully created
+          return
+        } else if (backupCr.status.state === 'Failed') {
+          throw new Error(`Backup '${backupCrName}' failed: ${backupCr.status.message}`)
+        }
+        // Wait more
+      }
+
+      await this.sleep(delayMs)
+      totalTimeMs += delayMs
+    }
+    throw new Error(`Backup CR '${backupCrName}' has not appeared in ${timeoutMs / 1000}s`)
+  }
+
   /**
    * Gets last 50 tags from the given repository.
    * @param repo repository name to list tag in
@@ -225,6 +287,43 @@ export class E2eHelper {
     return tags
   }
 
+  /**
+   * Returns list of sorted CSVs in stable OLM channel of Che Operator published in community-operators repository.
+   * The CSVs are sorted from the latest to oldest.
+   */
+  private async listOLMReleasedCSV(): Promise<string[]> {
+    const response = await this.octokit.repos.getContent({
+      owner: 'redhat-openshift-ecosystem',
+      repo: 'community-operators-prod',
+      path: 'operators/eclipse-che',
+    })
+    // There is an array in the response as given path points to a directory
+    const data = response.data as {name: string}[]
+    const csvsNames: string[] = []
+    data.map(item => csvsNames.push(item.name))
+    const stableChannelCSVNames = csvsNames.filter(csvName => {
+      // Filter versions like 7.37.2-all-namespaces
+      if (csvName.indexOf('-') !== -1) {
+        return false
+      }
+
+      // Filter non x.y.z
+      const versionParts = csvName.split('.')
+      if (versionParts.length !== 3) {
+        return false
+      }
+      // Ensure each part is a number
+      for (const versionPart of versionParts) {
+        if (!versionPart.match(/^[0-9]+$/)) {
+          return false
+        }
+      }
+      return true
+    })
+
+    return stableChannelCSVNames.sort((verA: string, verB: string) => semver.lt(verA, verB) ? 1 : -1)
+  }
+
   /**
    * Get previous version from crwctl repository
    */
@@ -234,6 +333,25 @@ export class E2eHelper {
     return latestTag.name
   }
 
+  /**
+   * Gets pre-latest and latest released version from crwctl repository
+   */
+   async getTwoLatestReleasedVersions(installer: string): Promise<[string, string]> {
+     if (installer === 'olm') {
+      // OLM installer uses community-operators marketplace.
+      // To list available versions see the following folder:
+      // https://github.com/redhat-openshift-ecosystem/community-operators-prod/tree/main/operators/eclipse-che
+      const csvs = await this.listOLMReleasedCSV()
+      return [csvs[1], csvs[0]]
+     } else if (installer === 'operator') {
+       // Operator installer uses templates from GitHub releases, which are marked with tags
+       const githubClient = new CheGithubClient()
+       const latestTags = (githubClient as any).sortSemanticTags(await this.listLatestTags(CHECTL_REPONAME))
+       return [latestTags[1].name, latestTags[0].name]
+     }
+     throw new Error(`Failed to get latest versions: unknown installer '${installer}'`)
+  }
+
   /**
    * Check if VERSION file exists and return content. In case if Version file doesn't exists
    * We are not in release branch and return `next`
